#!/usr/bin/bash 
################################################################################
#
# Copyright (c) 2020 Baidu.com, Inc. All Rights Reserved
#
################################################################################

# EasyPack私有化部署故障排查工具

# @Time    : 2023/10/19
# @File    : trouble_shooting
# @Author  : qlb
# @Version : ga_3.1

# 将脚本执行结果写入文件和终端屏幕
exec > >(tee trouble_shooting.log.txt) 2>&1 

pwd=$(cd `dirname $0`; pwd)
cat=$(command -v cat)
jq=$(command -v jq)
jq_extra=${pwd}/jq
jq=${jq:-$jq_extra}
# jq=${jq_extra}
# 配置信息,一般情况下不需要修改,保留默认值即可。
MODULE_JSON='/etc/private_cloud/conf/module.json'
WORK_DIR='/home/baidu/work'
HAS_CHECKED_AIPE_SERVER_SECURITY_IN_CONTAINER=false
NO=0
ERR_NO=0
res_array=()
# functions
# 高亮显示
green_echo() {
    echo -e "SUCCESS:\033[32m$@\033[0m"
}
red_echo() {
    echo -e "FAILED:\033[31m$@\033[0m"
}
yellow_echo() {
    echo -e "NOTICE:\033[33m$@\033[0m"
}
child_echo() {
    echo -e "NOTICE:\033[35m$@\033[0m"
}
start_echo(){
    echo -e "START:\033[35m$@\033[0m"
}
finished_echo() {
    echo -e "FINISHED:\033[35m$@\033[0m"
}

####
#Brief:
#  格式化输出
#Globals:
#  none
#Agruments:
#  task
#Returns:
#  none
####
function format_output_before() {
    local task=$1
    NO=$(($NO+1))
    echo 
    echo -e "TASK $NO. [${task}] ************************************************************************************************\n"
}

# function format_output_after() {
#     echo 
# }

####
#Brief:
#  判断jq是否安装
#Globals:
#  none
#Agruments:
#  none
#Returns:
#  none
####
function check_jq_is_already_installed() {
    command -v jq >/dev/null 2>&1     
    if [[ $? -ne 0 ]]; then
        # echo "jq not installed yet.尝试使用工具包内自带jq"
        [[ ! -e "${pwd}/jq" ]] && red_echo "未找到jq,请重新解压trouble_shooting.tar" && exit -1       
    fi
  
}

####
#Brief:
#  检查离线鉴权服务是否正常
#Globals:
#  none
#Agruments:
#  none
#Returns:
#  none
####
function check_offline_security_server() {   
    format_output_before "检查本机离线鉴权服务" 
    # 检查离线鉴权服务的版本号
    echo "当前离线鉴权服务版本："
    ${jq} '.modules[]|select(.name == "c-offline-security-server")|{name,nativeVersion}' $MODULE_JSON         
    # 检查当前服务器环境是否为虚拟化环境 
    if [[ -n `command -v systemd-detect-virt` ]]; then 
        if [[ x`systemd-detect-virt` == x"none" ]] ; then
            echo "服务器环境检查：物理机"
        else 
            echo "服务器环境检查："`systemd-detect-virt`
        fi 
    fi 
        
    if [[ -e "${WORK_DIR}/c-offline-security-server/conf/server.conf" ]]; then 
        security_port_this_machine=`${cat} ${WORK_DIR}/c-offline-security-server/conf/server.conf|awk '/port/'| awk -F ':' '{print $NF}'`
        security_host_this_machine=`${cat} ${WORK_DIR}/c-offline-security-server/conf/server.conf|awk '/AIPE_SEC_DT_IP/'| awk -F ':' '{print $NF}'`
    else 
        red_echo "${WORK_DIR}/c-offline-security-server/ 工作目录文件不完整，请重新部署 c-offline-security-server 模块"
        res_array+=("${WORK_DIR}/c-offline-security-server/ 工作目录文件不完整，请重新部署 c-offline-security-server 模块")
        return 
    fi
    ## 校验ip地址是否存在
    command -v ifconfig >/dev/null 2>&1
    if [[ $? -eq 0 ]]; then
        ifconfig | grep ${security_host_this_machine} -A 5 -B 2 >/dev/null 2>&1
        [[ $? != 0 ]] && red_echo "IP地址${security_host_this_machine}不存在,请卸载c-offline-security-server后重新部署" && \
        res_array+=("IP地址${security_host_this_machine}不存在,请卸载c-offline-security-server后重新部署") \
        && return 
       
    fi 
             
    ## 检查鉴权进程和端口号
    [[ $(ps -ef |grep auth_server |grep -v grep |wc -l)  -ge 2 ]] && [[ $(netstat -lnp |grep -E ":${security_port_this_machine}|:7091|:7092"| wc -l) -eq 3 ]] && \
        [[ $(netstat -lnp |grep -E ":${security_port_this_machine}|:7091|:7092"|awk '{print $NF}'|cut -d/ -f2|sort|uniq|wc -l) -eq 1 ]] 
    if [[ $? == 0 ]]; then
        green_echo "c-offline-security-server 离线鉴权服务进程和端口号${security_port_this_machine}|7091|7092全部正常"
        ## 检查鉴权license有效期   
        local license_api_status=$(curl -o /dev/null -s -w \"%{http_code}\" --connect-timeout 5 -m 10 ${security_host_this_machine}:${security_port_this_machine}/security/license)                   
        if [[ x${license_api_status} == x"\"200\"" ]]; then
            end_time=$(curl -s ${security_host_this_machine}:${security_port_this_machine}/security/license|${jq} .productLicenses[].notAfter -r |awk '{print $1 "-" $2}'|sort -n -k 1 -k 2 -k 3  -t '-'|awk -F'-' '{print $1$2$3}'|uniq|head -n 1)        
            if [[ $(date '+%Y%m%d') -ge ${end_time} ]]; then 
                red_echo "license已过期，详情如下："
                curl -s ${security_host_this_machine}:${security_port_this_machine}/security/license|${jq} '.productLicenses[]|{module: .subject, start:.notBefore, end:.notAfter}'
                res_array+=("license已过期,请往上浏览具体起始终止时间")
            else
                green_echo "license在有效期内"
                curl -s ${security_host_this_machine}:${security_port_this_machine}/security/license|${jq} '.productLicenses[]|{module: .subject, start:.notBefore, end:.notAfter}'
            fi
        else
            red_echo "/security/license接口调用失败" 
            res_array+=("/security/license接口调用失败")
        fi                                   
    else
        red_echo "离线鉴权服务异常，请核实相关进程以及端口号占用情况" && echo "进程信息：" && \
        res_array+=("离线鉴权服务异常，请往上浏览核实相关进程以及端口号占用情况") && \
        ps -ef |grep auth_server && echo "端口占用情况：" && netstat -lnp |grep -E ":${security_port_this_machine}|:7091|:7092"
    fi
  
    ## 检查鉴权日志
    ## 检查鉴权启动日志
    echo "鉴权启动日志/home/baidu/work/c-offline-security-server/log/auth_server.log如下："     
    # auth_server有文件大小上限，所以以下命令有可能返回空                    
    auth_server_log=`${cat} ${WORK_DIR}/c-offline-security-server/log/auth_server.log |grep -Ei 'warning|error|FATAL' |grep -Ev 'gethostbyname_r|8877|brpc.baidu.com|open log_r'|tail -n 5`    
    if [[ ! -n "${auth_server_log}" ]] ; then 
        echo "无有效日志，忽略。"
    else
        ${cat} ${WORK_DIR}/c-offline-security-server/log/auth_server.log |grep -Ei 'warning|error|FATAL' |grep -Ev 'gethostbyname_r|8877|brpc.baidu.com|open log_r'|tail -n 5
    fi 
    ## 检查容器内的鉴权客户端和容器外的鉴权服务端通信连接日志    
    AIPE_ERROR=$(${cat} ${WORK_DIR}/c-offline-security-server/log/aipe_info_$(date +%Y-%m-%d).log|grep -i 'error')    
    if [ -n "${AIPE_ERROR}" ] ; then 
        red_echo "鉴权连接日志/home/baidu/work/c-offline-security-server/log/aipe_info_$(date +%Y-%m-%d).log发现异常，如果是历史日志，请忽略"        
        res_array+=("鉴权连接日志发现异常，请检查日志时间，核实是否是历史日志")
        ${cat} ${WORK_DIR}/c-offline-security-server/log/aipe_info_$(date +%Y-%m-%d).log|grep -i 'error'|tail -n 5
    else
        green_echo "鉴权连接日志暂无异常"
        ${cat} ${WORK_DIR}/c-offline-security-server/log/aipe_info_$(date +%Y-%m-%d).log|grep -Ev 'gethostbyname_r|8877|brpc.baidu.com'|tail -n 5        
    fi  
           
    # format_output_after
}

####
#Brief:
#  检查docker启动的容器信息
#Globals:
#  none
#Agruments:
#  installed_models
#Returns:
#  none
####
function check_containers() {
    local installed_models=$@    
    format_output_before "检查容器状态"
    ## 打印容器列表,兼容人脸应用 模块名和容器名不一致的问题
    printf "NAMES\tPORTS\tNETWORKS\tSTATUS\n"
    docker ps -a --no-trunc --format "table {{.Names}}\t{{.Ports}}\t{{.Networks}}\t{{.Status}}" | grep -E `echo ${installed_models}|tr ' ' '|'|sed 's/face-server/face-service/'` 
    # format_output_after
    for one_container in ${installed_models}; 
    do        
        check_one_container ${one_container} 
    done     
}

####
#Brief:
#  检查容器网络的信息
#Globals:
#  none
#Agruments:
#  鉴权端口号
#Returns:
#  none
####
function check_docker_network() {    
    ## 打印基础信息        
    echo '网络设备:'
    ifconfig docker0
    echo 'docker network:'
    docker network ls
    echo "iptables规则（检查$1、7091、7092端口放行情况）:"
  
    iptables -nL|grep -E "$1|7091|7092"  
}


####
#Brief:
#  检查单个容器的信息
#Globals:
#  none
#Agruments:
#  one_container
#Returns:
#  none
####
function check_one_container() {
    local one_container=$1
    local docker_info
    local running_container
    local container_name
    local container_port
    local host_port
    local this_module_container_count
    local input_container_name
    # 兼容人脸识别应用
    [[ x${one_container} == x"face-server" ]] && one_container="face-service"
    
    # local network_connection_check_pass=false
    docker_info=$(docker ps -a --no-trunc  --filter "name=${one_container}" --format "table {{.Names}}\t{{.Ports}}"|awk 'NR>1{print}')   
    [[ ! -n "${docker_info}" ]] && return  

    # 校验是否部署多个实例
    this_module_container_count=$(docker ps -a --no-trunc  --filter "name=${one_container}" --format "table {{.Names}}\t{{.Ports}}"|awk 'NR>1{print}'|wc -l) 
    if [[ $this_module_container_count -gt 1 ]]; then 
        yellow_echo "匹配到与${one_container}相关的多个容器，如下："
        docker ps -a --no-trunc  --filter "name=${one_container}" --format "table {{.Names}}"|awk 'NR>1{print}'
        while : 
        do
            read -p "请选择一个容器名称输入: " input_container_name
            if [ -z $input_container_name ];then             #判断用户是否输入，如果未输入则打印error
                echo "输入为空，请重新输入"
                continue 
            else 
            # 精确匹配输入的容器名是否存在
                if [[ -z $(docker ps -a --no-trunc  --filter "name=^/${input_container_name}$" --format "table {{.Names}}\t{{.Ports}}"|awk 'NR>1{print}') ]]; then 
                    red_echo "该容器名${input_container_name} 不存在，请重新输入"
                    continue 
                else 
                    break 
                fi 
            fi                
        done         
    fi 
    
    format_output_before "检查 ${one_container} 容器应用服务"        
    ## 检查接口返回结果
    container_name=`echo ${docker_info}|cut -d' ' -f1`
    container_name=${input_container_name:-$container_name} 
     
    IFS=$'\n'
    running_container=$(docker ps -a --no-trunc  --filter "name=^/${container_name}$" --filter "status=running" --format "table {{.Names}}\t{{.Ports}}"|awk 'NR>1{print}')
        
    if [[ -n "${running_container}" ]]; then 
        docker_info=$(echo ${running_container} |tr -s ' ')                
        if [[ -n `echo ${docker_info}|cut -d' ' -f2` ]]; then 
            container_port=`echo ${docker_info}|cut -d' ' -f2|awk -F'[/ >]' '{print $(NF-1)}'`
            host_port=`echo ${docker_info}|cut -d' ' -f2|awk -F'[: -]' '{print $(NF-1)}'`
        fi                 
    else        
        output_container_log ${container_name}
        return         
    fi 
    
    IFS=$''

    # 跳过对mysql 、une-engine-biz、redis容器的网络检查
    if [[ x${one_container} == x"mysql" ]] || [[ x${one_container} == x"une-engine-biz" ]] || [[ x${one_container} == x"redis" ]]; then         
        return 
    fi 
    ## 人脸应用特殊检测
    if [[ x${one_container} == x"face-service" ]]; then 
        face_server_special_check ${container_name}     
    fi  
    aipe_server_security_host_in_operator=`docker inspect ${container_name} -f '{{ index (index .Config.Env) }}' | tr ' ' '\n'|grep "AIPE_SECURITY_SERVER_HOST"| sed 's/^.*=//'`
    
    ## 获取算子配置的鉴权节点端口号    
    aipe_server_security_port_in_operator_line=`docker inspect ${container_name} -f '{{ index (index .Config.Env) }}' | tr ' ' '\n'|grep "AIPE_SECURITY_SERVER_PORT"| sed 's/^.*=//'`
    
    aipe_server_security_port_in_operator=${aipe_server_security_port_in_operator_line:-8443}  # 赋默认端口号8443

    # 验证容器内是否含有curl命令
    # docker exec -it ${container_name} command -v curl >/dev/null
    # if [[ $? -ne 0 ]]; then 
    #     yellow_echo "容器内未找到/bin/curl命令，跳过容器内端口探活与网络联通性测试"
    #     ## 测试在宿主机通过HostIp:HostPort访问应用服务网络连通性 (Host -> Docker-Proxy -> Container)
    #     check_one_container_service_outside ${host_port}
    #     ## 检查AI容器日志
    #     output_container_log ${container_name}
    
    #     # format_output_after
    #     return                
    # fi 

    child_echo "您的私有化应用 ${one_container} 端口号为${host_port},调用接口时请使用该端口号" 
    echo "========网络联通性检测========"               
    ## 测试在容器内通过ContainerIp:ContainerPort请求应用服务网络连通性
    check_one_container_service_inside ${container_name} ${container_port}

    ## 测试在宿主机通过HostIp:HostPort访问应用服务网络连通性 (Host -> Docker-Proxy -> Container)
    check_one_container_service_outside ${host_port}
    
    if [[ $? == 1 ]]; then 
        echo "正在检查容器网络"
        check_docker_network $aipe_server_security_port_in_operator
        # yellow_echo "如您对宿主机的端口号做过调整，请手工检查端口号是否映射错误"
    fi

    if [[ x${HAS_CHECKED_AIPE_SERVER_SECURITY_IN_CONTAINER} == x"false" ]] && [[ -n ${aipe_server_security_host_in_operator} ]]; then                               
        if [[ ${aipe_server_security_host_in_operator} =~ "127.0.0.1" ]] || [[ ${aipe_server_security_host_in_operator} =~ "localhost" ]]; then 
            yellow_echo "检测到您填写的IP地址为${aipe_server_security_host_in_operator},如果没有切换容器网络为Host主机模式，需要修改为真实的地址"
        fi
        check_offline_security_server_in_container ${aipe_server_security_host_in_operator} ${aipe_server_security_port_in_operator} ${container_name} 
    fi

      

    ## 检查AI容器日志
    output_container_log ${container_name}
   
    # format_output_after
    
}

function check_one_container_service_inside() {
    local container_name=$1
    local container_port=$2
    local networkmode
    local service_port
    local wget_check
    if [[ ! -n ${container_port} ]]; then 
        # 判断网络模式
        networkmode=`docker inspect ${container_name}  -f '{{ index (index .HostConfig.NetworkMode) }}'`
        if [[ x${networkmode} == x"host" ]]; then 
            # 兼容iOCR的iocr-web 和iocr-api服务
            service_port=`docker inspect ${container_name} -f '{{ index (index .Config.Env) }}' | tr ' ' '\n'|grep "SERVICE_PORT"| sed 's/^.*=//'`
            if [[ -n ${service_port} ]]; then 
                container_port=${service_port}
            else 
                # yellow_echo "请执行 docker logs -f ${container_name} , 手工检查${container_name}服务"
                return 
            fi 
        else 
            return 
        fi 
    fi 
    # 判读curl是否存在
    docker exec -it ${container_name} command -v curl >/dev/null
    if [[ $? -ne 0 ]]; then 
        wget_check=`docker exec -i ${container_name} wget --delete-after -t1 -T1 http://0.0.0.0:${container_port} 2>&1`
        case $wget_check in 
            *"connected"*) ai_process_status="\"200\"";;
            *) ai_process_status="\"502\"";;
        esac
    else
        ai_process_status=$(docker exec -it ${container_name} curl -o /dev/null -s -w \"%{http_code}\" --connect-timeout 5 -m 10 0.0.0.0:${container_port})                 
    fi 

    if [[ x${ai_process_status} == x"\"200\"" ]] || [[ x${ai_process_status} == x"\"404\"" ]] || [[ x${ai_process_status} == x"\"302\"" ]] ; then 
        green_echo "容器端口探活：${container_name} 内 ContainerPort(${container_port}) 端口正常"
    else
        red_echo "容器端口探活： ${container_name} 内 ContainerPort(${container_port}) 拒绝提供服务，请稍后重试"
        res_array+=("容器端口探活： ${container_name} 内 ContainerPort(${container_port}) 拒绝提供服务，请稍后重试")
        
    fi 

      

}

function check_one_container_service_outside() {
    local host_port=$1   
    [[ ! -n ${host_port} ]] && return  
    host_to_container_status=$(curl -o /dev/null -s -w \"%{http_code}\" --connect-timeout 5 -m 10 0.0.0.0:${host_port}/status)
    if [[ x${host_to_container_status} == x"\"200\"" ]] || [[ x${host_to_container_status} == x"\"404\"" ]] || [[ x${host_to_container_status} == x"\"302\"" ]] ; then 
        green_echo "宿主机端口探活：宿主机 HostPort(${host_port}) 端口正常"
        return 0                                
    else
        red_echo "宿主机端口探活：宿主机 HostPort(${host_port}) 拒绝提供服务"
        res_array+=("宿主机端口探活：宿主机 HostPort(${host_port}) 拒绝提供服务")        
        return 1                  
    fi
    

}

function check_offline_security_server_in_container() {
    local aipe_server_security_host_in_operator=$1
    local aipe_server_security_port_in_operator=$2
    local container_name=$3
    local wget_check
    [[ ! -n ${aipe_server_security_host_in_operator} ]] && return 
    # echo "填写的鉴权地址为：${aipe_server_security_host_in_operator}" 
    OLD_IFS="$IFS"
    IFS=","
    array=(${aipe_server_security_host_in_operator})  
    IFS="$OLD_IFS"      
    for each_aipe_server_security_host_in_operator in ${array[@]};     
        do 
            ## 测试在容器内访问宿主机鉴权服务网络连通性 （Container -> Host）
            # 判断curl是否存在
            docker exec -it ${container_name} command -v curl >/dev/null
            if [[ $? -ne 0 ]]; then 
                wget_check=`docker exec -i ${container_name} wget --delete-after -t1 -T1 ${each_aipe_server_security_host_in_operator}:${aipe_server_security_port_in_operator}/security/license 2>&1`
                case $wget_check in 
                    *"connected"*) container_to_host_status="\"200\"";;
                    *) container_to_host_status="\"502\"";;
                esac
            else
                container_to_host_status=$(docker exec -it ${container_name} curl  -o /dev/null -s -w \"%{http_code}\" --connect-timeout 5 -m 10  ${each_aipe_server_security_host_in_operator}:${aipe_server_security_port_in_operator}/security/license)                            
            fi 
                            
            # container_to_host_status=$(docker exec -it ${container_name} curl  -o /dev/null -s -w \"%{http_code}\" --connect-timeout 5 -m 10  ${each_aipe_server_security_host_in_operator}:${aipe_server_security_port_in_operator}/security/license)        
            if [[ x${container_to_host_status} == x"\"200\"" ]]; then 
                green_echo "容器与外部鉴权服务网络联通性探测：容器 ${container_name} 与 ${each_aipe_server_security_host_in_operator}:${aipe_server_security_port_in_operator}鉴权服务通信正常"                            
            else
                red_echo "容器与外部鉴权服务网络联通性探测：在容器 ${container_name} 内请求 ${each_aipe_server_security_host_in_operator}:${aipe_server_security_port_in_operator}鉴权服务失败,请检查${each_aipe_server_security_host_in_operator} 节点鉴权服务是否正常或者防火墙规则是否放行8443端口"                                             
                res_array+=("容器与外部鉴权服务网络联通性探测：在容器 ${container_name} 内请求 ${each_aipe_server_security_host_in_operator}:${aipe_server_security_port_in_operator}鉴权服务失败,请检查${each_aipe_server_security_host_in_operator} 节点鉴权服务是否正常或者防火墙规则是否放行8443端口")
            fi            
        done
    HAS_CHECKED_AIPE_SERVER_SECURITY_IN_CONTAINER=true     
}

function output_container_log() {
    local container_name=$1
    ## 检查AI容器日志
    # docker logs ${container_name} |grep -Ev 'gethostbyname_r|8877|brpc.baidu.com' |tail -n 300 > result.txt 
    yellow_echo "AI应用容器 ${container_name} 日志如下"
    echo "========== fail|Error ======="
    docker logs ${container_name} |grep -Ev 'gethostbyname_r|8877|brpc.baidu.com'|grep -Ei 'fail|Error|Out of memory' -C 5|tail -n 25
    echo "========== Latest 10 =========="
    docker logs ${container_name} |grep -Ev 'gethostbyname_r|8877|brpc.baidu.com'|tail -n 10
    # echo "docker logs ${container_name} |grep -Ev 'gethostbyname_r|8877|brpc.baidu.com'|tail -n 10"

}

####
#Brief:
#  人脸识别单独检测
#Globals:
#  none
#Agruments:
#  none
#Returns:
#  none
####
function face_server_special_check() {
    local container_name=$1     
    ## 外部mysql连通性检测
    # 读取mysql的ip地址
    MYSQL_SVC_SERVICE_HOST=`${cat} ${WORK_DIR}/face-server/project-conf/sconf/service.conf|awk '/MYSQL_SVC_SERVICE_HOST/'| awk -F '=' '{print $NF}'`
    MYSQL_SVC_SERVICE_PORT=`${cat} ${WORK_DIR}/face-server/project-conf/sconf/service.conf|awk '/MYSQL_SVC_SERVICE_PORT/'| awk -F '=' '{print $NF}'`
    [[ -z ${MYSQL_SVC_SERVICE_HOST} ]] && red_echo "未解析到mysql地址，请先参考部署文档安装mysql……" && \
    res_array+=("未解析到mysql地址，请先参考部署文档安装mysql……") && return 
    MYSQL_SVC_SERVICE_PORT=${MYSQL_SVC_SERVICE_PORT:-5535}
    
    # echo  "=====检查/etc/rc.local中 mysql和 c-offline-security-server 启动顺序====="
    rc_local_path=$(systemctl cat rc-local.service |grep "Description="|awk -F= '{print $2}'|awk '{print $1}')
    # 找不到，则默认为/etc/rc.local
    rc_local_path=${rc_local_path:-/etc/rc.local}
    mysql_num_in_rc=`grep -n '/home/idl-face/databus5535/sh/mysql_boot.sh' ${rc_local_path}|awk -F: '{print $1}'`
    c_offline_security_server_num_in_rc=`grep -n '/home/baidu/work/op/module_boot.sh c-offline-security-server' ${rc_local_path}|awk -F: '{print $1}'`
    [[ -z $mysql_num_in_rc ]] && red_echo "未找到mysql开机启动命令... 请将bash /home/idl-face/databus5535/sh/mysql_boot.sh 添加到${rc_local_path}" && return 
    [[ -z $c_offline_security_server_num_in_rc ]] && red_echo "未找到c-offline-security-server 开机启动命令……，请将. /home/baidu/work/op/module_boot.sh c-offline-security-server 添加到${rc_local_path} (注意. 后有空格)" && return 
    if [[ $mysql_num_in_rc -gt $c_offline_security_server_num_in_rc ]]; then 
        red_echo "
            mysql 和 c-offline-security-server 开机启动顺序异常，请手动调整启动顺序,正确顺序为：
            bash /home/idl-face/databus5535/sh/mysql_boot.sh 在前
            . /home/baidu/work/op/module_boot.sh c-offline-security-server 在后 
        "
        res_array+=("mysql 和 c-offline-security-server 开机启动顺序异常,请往上浏览，按照要求调整顺序")
    fi 

    docker exec -it ${container_name} curl -o /dev/null -s ${MYSQL_SVC_SERVICE_HOST}:${MYSQL_SVC_SERVICE_PORT}
    if [[ $? -eq 0 ]]; then 
        green_echo "容器内连接宿主机mysql服务正常..."
    else 
        red_echo "容器内连接宿主机mysql服务失败..."    
        echo "检查databus5535(mysql)进程：" 
        [[ -z $(ps -ef |grep databus5535 |grep -v grep) ]] && red_echo "未检测到databus5535进程，请确认是否已参考官网文档部署了数据库服务，如已安装请执行该命令手动启动mysql: bash /home/idl-face/databus5535/sh/mysql_boot.sh"
        echo "iptables规则：" 
        iptables -nL|grep ${MYSQL_SVC_SERVICE_PORT} 
        if [[ $(iptables -nL|grep ${MYSQL_SVC_SERVICE_PORT}|wc -l) -eq 0 ]]; then
            red_echo "请执行该命令放行该端口号: iptables -I INPUT -p tcp --dport 5535 -j ACCEPT   另外需将其追加到${rc_local_path}文件中"
        fi 
        res_array+=("容器内连接宿主机mysql服务失败...,请往上浏览按照解决方案来处理")

    fi 

    # 人脸识别单独检测-face-general-service 检查
    face_general_service_check ${container_name} 

    # 数据库连接日志查看
    echo "========查看数据库连接日志========"
    docker exec -it ${container_name} bash -c "cat /home/idl-face/public_bridge/log/bridge.INFO|grep -v '^[WI]'|tail -n 5"
    
    
    # 检查人脸服务日志
    # echo "========查看人脸服务日志========"
    # docker exec -it ${container_name} bash -c "cat /home/idl-face/feature-frame/log/service.log.wf |grep -Ev 'log_r|log.wf'|tail -n 5"
    
    

}

####
#Brief:
#  人脸识别单独检测-face-general-service 检查
#Globals:
#  none
#Agruments:
#  none
#Returns:
#  none
####
function face_general_service_check() {
    local container_name=$1    
    while [[ $(docker exec -i ${container_name} bash -c '[[ ! -f sys_start.log ]] && echo true') == true ]]  
    do 
        yellow_echo "人脸应用服务正在启动中……,预计最长需要5～10min左右，请您耐心等待,继续进行检测中，请勿中断……"
        sleep 5
    done 
   
    while true        
        do 
            if [[ $(docker exec -i ${container_name} bash -c '[[ $(tail -n 1 /home/idl-face/sys_start.log) =~ hhvm.*ok ]] && echo true') != true ]]
            then 
                yellow_echo "人脸应用服务正在启动中……,预计最长需要5～10min左右，请您耐心等待,继续进行检测中，请勿中断……";
                sleep 15;
            else 
                echo "人脸应用服务启动结束";
                 # 首先，我们使用tac命令将log文件的行倒序排列
                # 然后我们使用awk命令来找到出现"Starting hhvm:.ok"的行号
                # 我们把第一个和第二个出现的行号存储下来，然后输出这两个行号之间的所有行,在文件中执行，要将$0转义，避免识别为shell脚本文件名
                # 最后我们再把所有行再次倒序排列以回到原始顺序
                log=$(docker exec -i ${container_name} bash -c "tac sys_start.log | awk '/hhvm:.*ok/{c++; if(c==2){exit}} {line[NR]=\$0} END{for(i=1;i<=NR;i++) print line[i]}' | tac")
                if [[ $log == *"fail"* ]]; then
                    red_echo "部分人脸服务启动失败,开始盲检face-general-service"
                    
                    # 在docker容器中监听debug结果并设置20s超时时间，超时后自动终止docker容器内进程,此处非宿主机进程，可通过打印的PID来验证
                    docker exec -i ${container_name} /bin/bash << EOI
                    cd /home/idl-face/aikl-face-package && bin/face-general-service_control fgstart & 
                    pid=\$!
                    echo "PID: \$pid" # 输出PID的值
                    sleep 20
                    if ps -p \$pid > /dev/null
                    then 
                    kill \$pid
                    fi
EOI
                    res_array+=("人脸服务face-general-service可能启动失败，请浏览上述日志继续定位")
                else
                    green_echo "人脸容器内服务启动成功"
                fi
                break 
            fi                             
        done 

}    


####
#Brief:
#  检查ulimit信息
#Globals:
#  none
#Agruments:
#  none
#Returns:
#  none
####
function check_ulimit(){
    # ulimit控制的维度是shell会话或shell创建的进程（至少对于nofile来说）。即：当前用户打开的文件数，是可以远远超过nofile的值。   
    # losf命令虽然作用是"list open files"，但用lsof | wc -l统计打开文件数上非常不准确。主要原因是：
    # 某些情况下，一行可能显示的是线程，而不是进程，对于多线程的情况，就会误以为一个文件被重复打开了很多次
    # 子进程会共享file handler
    
    ## 直接查看/proc/sys/fs/file-nr（file-nr说明参考：https://www.kernel.org/doc/Documentation/sysctl/fs.txt）。
    ## file-nr 中的三个值分别表示已分配的文件句柄数、已分配但未使用的文件句柄数和最大文件句柄数

    ## ulimit -n 用户级限制
    ## file-max 内核级硬限制
    ## https://unix.stackexchange.com/questions/476351/difference-between-ulimit-lsof-cat-proc-sys-fs-file-max
    # format_output_before "检查文件句柄数及inodes使用情况"
    # allocated_file_nr=`${cat} /proc/sys/fs/file-nr|awk '{print $1}'`
    # allocated_file_max=`${cat} /proc/sys/fs/file-max`
    user_max_file_nr=`ulimit -n`
    # yellow_echo "系统所有进程可分配的最大文件句柄数:${allocated_file_max}, 已分配的文件句柄数: ${allocated_file_nr}"
    echo "当前用户open files阈值: ${user_max_file_nr}"
    if [[ ${user_max_file_nr} -le 1024 ]]; then 
        yellow_echo "建议将ulimit -n 调整为65535"
    fi 
    # echo "/home/baidu所在文件系统inodes使用情况"
    # df -i /home/baidu|awk '{print }'


    # format_output_after
}

####
#Brief:
#  检查系统自带模块的版本号
####
function check_sys_module_version(){
    # 一般包括nvidia和docker
    # 检查 nvidia-smi 是否存在
    if [[ $(command -v nvidia-smi) ]]; then
        #获取显卡驱动版本
        local driver_version=$(nvidia-smi --query-gpu=driver_version --format=csv,noheader | head -n 1)
        #获取 CUDA 版本
        local cuda_version=$(nvidia-smi | grep -i 'CUDA Version' | awk -F ': ' '{print $3}'|awk '{print $1}')
        #获取每一块显卡的类型和对应的编号
        local gpu_info=$(nvidia-smi --query-gpu=name --format=csv,noheader | awk '{print $0"("NR-1")" }' | tr '\n' ',')
        echo "GPU Driver Version: $driver_version, CUDA Driver Version: $cuda_version, GPU Info: $gpu_info"
    else
        echo "nvidia-smi not found"
    fi

    #检查 docker 是否存在
    if [[ $(command -v docker) ]]; then
        #获取 docker.service 的状态
        docker_service_status=$(systemctl is-active docker)
        #获取 Docker 版本
        docker_version=$(docker -v | awk '{print $3}' | sed 's/,//')
        echo "Docker Service Status: $docker_service_status, Docker Version: $docker_version"
    else
        echo "docker not found"
    fi 
    # 检查nvidia-docker是否存在
    if [[ $(command -v nvidia-docker) ]]; then
        nvidia_docker_version=$(nvidia-docker version|head -n 1)
        echo "$nvidia_docker_version"
    fi        
  
}

####
#Brief:
#  检查开机启动服务
####
function check_rc_local_healths(){
    service_status=$(systemctl is-active rc-local.service)
    if [ "$service_status" != "active" ]; then
        res_array+=("rc-local.service开机启动服务没有在运行, 请执行sudo systemctl start rc-local.service启动") 
        red_echo "rc-local.service开机启动服务没有在运行, 请执行以下命令来启动服务："        
        red_echo "sudo systemctl start rc-local.service"
    else 
        green_echo "rc-local.service is activate"    
    fi    
}
####
#Brief:
#  检查已经安装的模块
#Globals:
#  none
#Agruments:
#  none
#Returns:
#  none
####
function check_installed_modules() {    
    # 判断当前用户
    [[ $(whoami) != "root" ]] && red_echo "请切换至root用户执行" && exit -1
    check_jq_is_already_installed
    # 检查文件句柄数
    # format_output_before "检查文件句柄数"
    # check_ulimit
    # 已安装模块信息
    
    INSTALLED_MODULES=$(${jq} -r '.installed[]' $MODULE_JSON)  
    # 判断externalInstalled是否存在于json中
    jude_externalInstalled_exist=`${jq} -r '.externalInstalled' $MODULE_JSON`
    if [[ $jude_externalInstalled_exist != "null" ]]; then 
        EXTERNAL_INSTALLED_MODULES=$(${jq} -r '.externalInstalled[]' $MODULE_JSON) 
    fi         
    
    # 部分情况下人脸容器通过脚本安装失败，face-server未写入module.json中，此处补全
    if [[ -n $(docker ps --format '{{.Names}}' | grep "^face-service-[0-9]*-[0-9]*$") ]] ; then 
        if [[ ${INSTALLED_MODULES} != *"face-server"* ]]; then 
            INSTALLED_MODULES=$INSTALLED_MODULES" face-server"
        fi     
    fi 
    format_output_before "检查本机已经安装的模块"
    echo  "来自部署包的模块 : "${INSTALLED_MODULES}
    echo "来自操作系统自带的模块 : "${EXTERNAL_INSTALLED_MODULES}
    check_sys_module_version

    format_output_before "检查开机启动服务状态"
    check_rc_local_healths

    for one_module in ${INSTALLED_MODULES};
    do  
        # 检查鉴权  
        if [[ x${one_module} == x'c-offline-security-server' ]]; then            
            check_offline_security_server                  
        fi    
    done

    # 检查容器     
    check_containers ${INSTALLED_MODULES} 
    finished_echo "##########检测报告概览##########"
    if [[ ${#res_array[@]} == 0 ]]; then
        green_echo "检测无异常"
    else
        for element in ${res_array[@]}
        do  
            ERR_NO=$((ERR_NO+1))
            red_echo $ERR_NO". "${element}
        done 
    fi
    
    finished_echo "##############################"
    finished_echo "检测结束，请核实是否有异常（绿色表示正常、红色表示异常、黄色表示警告、紫色表示提示）。如果发现有错误日志，\n请核实是否为历史遗留日志（当前时间：$(date '+%Y%m%d %H:%M:%S %Z')），完整检测报告记录在trouble_shooting.log.txt文件中"
    # if [[ $(echo ${INSTALLED_MODULES} |tr -d "\n" ) =~ "face-server" ]]; then 
    #     finished_echo "人脸容器内服务完全启动需要等待一段时间，如检测到人脸端口探活失败，可等待5～10min后重新执行该脚本"
    # fi 
}

# main 
start_echo "###################这是检测报告的头部#############################"
check_installed_modules
finished_echo "##########如需截图请从检测报告头部开始截图##########"
